# -*- coding: utf-8 -*-
"""
/***************************************************************************
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                             -------------------
        begin                : 2022-10-25
        git sha              : $Format:%H$
        copyright            : (C) 2022 by LAGC-UCA
        email                : lagc@uca.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
import os.path
import csv
import processing
import numpy as np 
import sys
import time
from qgis.PyQt.QtCore import *
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import * 
from PyQt5.QtGui import * 
from PyQt5.QtCore import *
from qgis.core import *
from qgis.gui import * 
from mpl_toolkits.basemap import pyproj
from scipy.spatial import Delaunay
from .Strain_Stress_App_dialog import StrainStressAppDialog
from .RectangleMapTool import RectangleMapTool

#Display input data
class TableModel(QtCore.QAbstractTableModel):
    def __init__(self, data):
        super(TableModel, self).__init__()
        self._data = data
    def data(self, index, role):
        if role == Qt.DisplayRole:
            return self._data[index.row()][index.column()]
    def rowCount(self, index):
        return len(self._data)
    def columnCount(self, index):
        return len(self._data[0])

#Main Class
class StrainStressApp:
    def __init__(self, iface):
        self.iface = iface
        self.plugin_dir = os.path.dirname(__file__)
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'Strain_Stress_App_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        self.actions = []
        self.menu = self.tr(u'&Strain_Stress_App')
        self.first_start = None
        
        # Save the name of the grid interpolation output file
        self.fidout_name = ""
        self.path_input = ""

        # Save the limits of the frame of reference of the grid
        self.xmin = 0.0
        self.xmax = 0.0
        self.ymin = 0.0
        self.ymax = 0.0

        self.canvas = self.iface.mapCanvas()
                 
    def tr(self, message):
        return QCoreApplication.translate('Strain_Stress_App', message)

    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToVectorMenu(
                self.menu,
                action)
        self.actions.append(action)

        return action

    def initGui(self):
        icon_path = self.plugin_dir + '/icon.png' #Q-Str2-Models icon
        
        self.add_action(
            icon_path,
            text=self.tr(u'Q-Str2-Models'),
            callback=self.run,
            parent=self.iface.mainWindow())
        self.first_start = True

    def unload(self):
        for action in self.actions:
            self.iface.removePluginVectorMenu(
                self.tr(u'&Q-Str2-Models'),
                action)
            self.iface.removeToolBarIcon(action)
    
    #Select frame of reference (Study area) link to RectangleMapTool.py
    def select_MREF(self):
        self.canvas.setMapTool(self.toolRectangleMap)
       
        bufferLength = 1000
        polygonSides = 4
 
        points = []
 
        filas = self.dlg.filas.value()
        columnas = self.dlg.columnas.value()
        xlonmax = self.dlg.xlonmax.text().replace(",", ".")
        xlonmin = self.dlg.xlonmin.text().replace(",", ".")
        xlonmax = float(xlonmax) if xlonmax else 0
        xlonmin = float(xlonmin) if xlonmin else 0
        ylatmax = self.dlg.ylatmax.text().replace(",", ".")
        ylatmin = self.dlg.ylatmin.text().replace(",", ".")
        ylatmax = float(ylatmax) if ylatmax else 0
        ylatmin = float(ylatmin) if ylatmin else 0
        
        inc_x = (xlonmax - xlonmin)/columnas
        inc_y = (ylatmax - ylatmin)/filas
        
        x = self.dlg.xlonmin.text().replace(",", ".")
        y = self.dlg.ylatmin.text().replace(",", ".")
        
        x = float(x) if x else 0
        y = float(y) if y else 0
    
    #Loading the layers
    def carga_fidout(self):
       
        s = QSettings()
        oldValidation = s.value( "/Projections/defaultBehaviour")
        s.setValue( "/Projections/defaultBehaviour", "useGlobal" )
        
        InFlPth="file:///" + self.fidout_name
        InFlnm = os.path.basename(self.fidout_name)
                
        active_Layer_CRS=self.iface.mapCanvas().currentLayer().crs().authid()
      
        #Creator of final uri
        uri = InFlPth+"?delimiter=%s&xField=%s&yField=%s&crs=%s" % (",","x","y", active_Layer_CRS)
        bh = QgsVectorLayer(uri, InFlnm, "delimitedtext")

        #--- Confirm something is loaded and valid
        bh.isValid()

        #--Show the active layer in Qgis layer box
        QgsProject.instance().addMapLayer(bh)

        # #--- Turn CRS dialog box back on again
        s.setValue( "/Projections/defaultBehaviour", oldValidation )
    
    #Input file selector    	    
    def select_input_file(self):
      filename, _filter = QFileDialog.getOpenFileName(
        self.dlg, "Select entry file","", '*.*')
      self.dlg.fich_datos.setText(filename)
      with open(filename) as csvDataFile:
            csvReader = csv.DictReader(csvDataFile, delimiter=',') 
            filas = list (csvReader)
            cols = 3 
            este =  [row['EAST'] for row in filas if row['EAST']]
            norte = [row['NORTH'] for row in filas if row['NORTH']]
            up =    [row['UP'] for row in filas if row['UP']]
            numpy_array = np.array([este,norte,up])
            contenido_tabla=numpy_array.T
            self.model = TableModel(contenido_tabla.tolist())
            self.dlg.tabla.setModel(self.model) 

    #Output files selector
    def select_output_file(self):
      filename, _filter = QFileDialog.getSaveFileName(
        self.dlg, "Save file as","", '*.csv')
      self.dlg.fich_out.setText(filename)
         
    def select_ejecutar(self):
        self.iface.messageBar().pushMessage(
            "Create Grid", "Successfully done",
            level=Qgis.Success, duration=4)
        
        filas = self.dlg.filas.value()
        columnas = self.dlg.columnas.value()
        
        xlonmax = self.dlg.xlonmax.text().replace(",", ".")
        xlonmin = self.dlg.xlonmin.text().replace(",", ".")
        xlonmax = float(xlonmax) if xlonmax else 0
        xlonmin = float(xlonmin) if xlonmin else 0
        
        ylatmax = self.dlg.ylatmax.text().replace(",", ".")
        ylatmin = self.dlg.ylatmin.text().replace(",", ".")
        ylatmax = float(ylatmax) if ylatmax else 0
        ylatmin = float(ylatmin) if ylatmin else 0
        
        self.xmin = xlonmin
        self.xmax = xlonmax
        
        self.ymin = ylatmin
        self.ymax = ylatmax

        inc_x = (xlonmax - xlonmin)/columnas
        inc_y = (ylatmax - ylatmin)/filas
        
        x = self.dlg.xlonmin.text().replace(",", ".")
        y = self.dlg.ylatmin.text().replace(",", ".")
        
        x = float(x) if x else 0
        y = float(y) if y else 0
        
        for i in range(filas):
            for j in range(columnas):
                x += inc_x
            y += inc_y

    	# Selection of the input and output frame of reference
        epsgIN = "EPSG:"+self.dlg.epsgIN.currentText()
        epsgOUT = "EPSG:"+self.dlg.epsgOUT.currentText()
        
        #Input and output projections
        prjEntrada=pyproj.Proj("+init="+epsgIN)	
        prjSalida=pyproj.Proj("+init="+epsgOUT)	
        
        #Input and output paths
        path_entrada, f_entrada = os.path.split(self.dlg.fich_datos.text())
        self.path_input = path_entrada

        filesDatos=self.dlg.fich_datos.text()
        path_salida, f_salida = os.path.split(self.dlg.fich_out.text())
        nombresFiles=self.dlg.fich_out.text()

        # types of interpolation
        tipointer=int(self.dlg.interpolacion.currentIndex())

        #conversion to meters
        dias=365.0 
        pasoametros=1000.0
        unidades="mm"
        
        #Horizontal and/or vertical displacements
        hori=1.0
        vert=1.0
 
        xmin=xlonmin
        xmax=xlonmax
        ymin=ylatmin
        ymax=ylatmax

        if epsgIN !="EPSG:4326": #CRS initial condition
            xmin,ymin=prjEntrada(xmin,ymin,inverse=True)
            xmax,ymax=prjEntrada(xmax,ymax,inverse=True)
        xmin,ymin=prjSalida(xmin,ymin)
        xmax,ymax=prjSalida(xmax,ymax)

        #Data file header
        cabecera=["NAME","Y","X","H","EAST","NORTH","UP","SE","SN","SVU"]
        nombres=[cabecera[i] for i in range(len(cabecera))]
        
        #Data file formats
        cabecera=["S4","float","float","float","float","float","float","float","float","float"]
        formatos=[cabecera[i] for i in range(len(cabecera))]

        #field numbers in header
        camposite=0
        campos=[1, 2, 4, 5, 6] #X,Y,E,N,U
        camposxyenu=[campos[i] for i in range(len(campos))]

        if tipointer>2:
             sufijo='_ARN_'
        elif tipointer==2:
             sufijo='_EXP'
        elif tipointer==1:
             sufijo='_IDW'
        else:
             sufijo='_DELAUNAY'
        excluir=[]

        filedatos=filesDatos
        nomfil=nombresFiles+sufijo
        fd = open(filedatos,'r')    
        d = np.loadtxt(fd,
                   delimiter=',',
                   dtype={ 'names':(nombres),
                          'formats':(formatos)},
                   comments='%')
        fd.close()
        sites=np.array([ele[camposite] for ele in d if (ele[camposite] not in excluir)]).transpose()
        a=np.array([[ele[i] for ele in d if (ele[camposite] not in excluir)] for i in camposxyenu]).transpose()
    
        #Velocities file output "mm" 
        with open(path_salida+'/2_' + f_salida.split(".")[0] + '.csv', 'w+') as output_file:
             line = 'NAME,X,Y,EAST,NORTH,UP\n'
             output_file.write(line)

        fidout1 = open(path_salida+'/2_' + f_salida.split(".")[0] + '.csv','w+')
        fidout1.write('NAME,X,Y,EAST,NORTH,UP\n');
        
        for i in range(len(a)):
            if epsgIN !="EPSG:4326":
                a[i,0],a[i,1]=prjEntrada(a[i,0],a[i,1],inverse=True)
            a[i,0],a[i,1]=prjSalida(a[i,0],a[i,1])
            a[i,2]=a[i,2]*365*1000/pasoametros/dias
            a[i,3]=a[i,3]*365*1000/pasoametros/dias
            a[i,4]=a[i,4]*365*1000/pasoametros/dias
            fidout1.write('%s,%-8.2f,%-8.5f,%-8.5f,%-8.5f,%-8.5f\n'%\
				(sites[i],a[i,0],a[i,1],a[i,2],a[i,3],a[i,4]))
        fidout1.close()
    
        if tipointer!=0:
               intervalos_x = columnas-1
               intervalos_y = filas-1
               yint = abs(ymax-ymin)/intervalos_y
               xint = abs(xmax-xmin)/intervalos_x
               n=len(a)
               MAXGEO=np.zeros([intervalos_x+1,intervalos_y+1])
               EMAX=np.zeros([intervalos_x+1,intervalos_y+1])
               EMIN=np.zeros([intervalos_x+1,intervalos_y+1])
               DILATACION=np.zeros([intervalos_x+1,intervalos_y+1])
               CIZALLA=np.zeros([intervalos_x+1,intervalos_y+1])
               ROTATION=np.zeros([intervalos_x+1,intervalos_y+1])
               THETASTRAIN=np.zeros([intervalos_x+1,intervalos_y+1])
               INCLINACION=np.zeros([intervalos_x+1,intervalos_y+1])
               THETAINCLI=np.zeros([intervalos_x+1,intervalos_y+1])
               ELEVACION=np.zeros([intervalos_x+1,intervalos_y+1])
               #Results file output (file for raster draw)
               self.fidout_name = path_salida + '/1_' + f_salida.split(".")[0] + sufijo + '.csv'
               fidout = open(self.fidout_name,'w+')
               fidout.write('X,Y,EAST,NORTH,EMAX,EMIN,THETA,ROTATION,DILATATION,SHEAR,MAXGEO,EMAXABS,EMINABS,THETA90,THETA180,THETA270\n');
               X0=np.zeros([intervalos_x+1])
               Y0=np.zeros([intervalos_y+1])
               alpha=0
               s=0
               
               for u in range(n):
                   for v in range(u+1,n):
                       if (u!=v):
                           s=s+1
                           alpha=alpha+np.sqrt((a[u,1]-a[v,1])**2+(a[u,0]-a[v,0])**2)
               alpha=alpha/s
               alpha=2*np.sqrt((xint**2+yint**2))
               D=np.empty(shape=[0,14]) 
               for i in range(intervalos_x+1):
                   for j in range(intervalos_y+1):
                       x0=xmin+(i)*xint
                       y0=ymin+(j)*yint
                       distancias_malla=np.empty(shape=[0,2])
                       for k in range(n):
                           dist=np.sqrt((a[k,0]-x0)**2+(a[k,1]-y0)**2)
                           distancias_malla=np.append(distancias_malla,[[dist,k]],axis=0)
                       orden = distancias_malla.argsort(axis=0)
                       numpuntos=n 
                       if tipointer>2:
                           numpuntos=tipointer 
                  
                       rango=(orden[i,0] for i in range(numpuntos))
                       A=np.zeros([numpuntos*3,9])
                       V=np.zeros([numpuntos*3,1])
                       W=np.zeros([numpuntos*3,numpuntos*3])
                       s=0
                       for k in rango:
                           dist=np.sqrt((a[k,0]-x0)**2+(a[k,1]-y0)**2)
                    
                           if tipointer==2:
                               peso=np.exp(-dist**2/2/alpha**2)
                           else:
                               peso=1/dist**2
                           W[(s+1)*3-3,(s+1)*3-3]=peso
                           W[(s+1)*3-2,(s+1)*3-2]=peso
                           W[(s+1)*3-1,(s+1)*3-1]=peso
                           A[(s+1)*3-3,0]=a[k,0]-x0
                           A[(s+1)*3-3,1]=a[k,1]-y0
                           A[(s+1)*3-3,2]=1
                           A[(s+1)*3-2,3]=a[k,0]-x0
                           A[(s+1)*3-2,4]=a[k,1]-y0
                           A[(s+1)*3-2,5]=1
                           A[(s+1)*3-1,6]=a[k,0]-x0
                           A[(s+1)*3-1,7]=a[k,1]-y0
                           A[(s+1)*3-1,8]=1
                           V[(s+1)*3-3]=a[k,2]/1000
                           V[(s+1)*3-2]=a[k,3]/1000
                           V[(s+1)*3-1]=a[k,4]/1000
                           s=s+1
                       N=np.dot(np.dot(A.transpose(),W),A)
                       V=np.dot(np.dot(A.transpose(),W),V)
                       B=np.dot(np.asmatrix(N)**-1,V)
                       exx=B[0]
                       eyy=B[4]
                       exy=(B[1]+B[3])/2
                       rotation=(B[1]-B[3])/2
                       dilatacion=exx+eyy
                       cizalla1=exx-eyy
                       cizalla2=2*exy
                       maxcizalla=np.sqrt(cizalla1**2+cizalla2**2)
                       thetastrain=.5*np.arctan(-cizalla2/cizalla1) + np.pi/2
                       emax=(dilatacion+maxcizalla)/2
                       emin=(dilatacion-maxcizalla)/2
                       maxgeo=max(np.abs(emax),np.abs(emin))
                       taux=-B[6]
                       tauy=-B[7]
                       inclinacion=np.sqrt(taux**2+tauy**2)
                       thetaincli=np.arctan(tauy/taux)
                       #Output parameters: east, north, elevation, delta gamma emax and emin, fi-strain max-defor-geo, inclination, fi-inclination...
                       D=np.append(D,[[x0, y0, B[2]*1000, B[5]*1000, B[8]*1000, dilatacion, maxcizalla, emax, emin, thetastrain,  maxgeo, inclinacion, thetaincli, rotation]],axis=0)
                       X0[i]=x0
                       Y0[j]=y0
                       EMAX[j,i]=emax*10**9
                       EMIN[j,i]=emin*10**9
                       ROTATION[j,i]=rotation*10**9
                       DILATACION[j,i]=dilatacion*10**9
                       CIZALLA[j,i]=maxcizalla*10**9
                       THETASTRAIN[j,i]=thetastrain
                       INCLINACION[j,i]=inclinacion*10**9
                       THETAINCLI[j,i]=thetaincli*10**9
                       ELEVACION[j,i]=B[8]*1000 #mmilimetros
                       MAXGEO[j,i]=maxgeo*10**9#sqrt((B(3)*100)**2+(B(6)*100)**2+(B(9)*100)**2);
                       fidout.write('%-8.2f,%-8.2f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f\n'%\
    				(x0,y0,B[2]*1000, B[5]*1000,EMAX[j,i],EMIN[j,i],THETASTRAIN[j,i]*180.0/np.pi,ROTATION[j,i],DILATACION[j,i],CIZALLA[j,i],MAXGEO[j,i],np.abs(EMAX[j,i]),np.abs(EMIN[j,i]),90+THETASTRAIN[j,i]*180.0/np.pi,180+THETASTRAIN[j,i]*180.0/np.pi,270+THETASTRAIN[j,i]*180.0/np.pi))
               fidout.close()
               self.carga_fidout()
        else:
               puntos=np.array([[a[i,0],a[i,1]] for i in range(len(a))])
               tri=Delaunay(puntos)
               n=len(tri.simplices)
               #Results file output (file for raster draw) "Second Condition"
               self.fidout_name = path_salida + '/1_' + f_salida.split(".")[0] + sufijo + '.csv'
               fidout = open(self.fidout_name,'w+')
               fidout.write('X,Y,EAST,NORTH,EMAX,EMIN,THETA,ROTATION,DILATATION,SHEAR,MAXGEO,EMAXABS,EMINABS,THETA90,THETA180,THETA270\n');
               X0=np.zeros([n])
               Y0=np.zeros([n])
               s=0
               D=np.empty(shape=[0,14]) 
               x=np.zeros(n)
               y=np.zeros(n)
               for i in range(n):
                   x[0]=puntos[tri.simplices[i,0]][0]
                   y[0]=puntos[tri.simplices[i,0]][1]    
                   x[1]=puntos[tri.simplices[i,1]][0]
                   y[1]=puntos[tri.simplices[i,1]][1]    
                   x[2]=puntos[tri.simplices[i,2]][0]
                   y[2]=puntos[tri.simplices[i,2]][1]    
                   x0=(x[0]+x[1]+x[2])/3
                   y0=(y[0]+y[1]+y[2])/3
                   s=0
                   A=np.zeros([6,6])
                   V=np.zeros([6,1])
                   s=0
                   for k in [0,1,2]:
                       A[(s+1)*2-2,0]=x[k]-x0
                       A[(s+1)*2-2,1]=y[k]-y0
                       A[(s+1)*2-2,2]=1
                       A[(s+1)*2-1,3]=x[k]-x0
                       A[(s+1)*2-1,4]=y[k]-y0
                       A[(s+1)*2-1,5]=1
                       V[(s+1)*2-2]=a[tri.simplices[i,k],2]/1000
                       V[(s+1)*2-1]=a[tri.simplices[i,k],3]/1000
                       s=s+1
                   N=np.dot(A.transpose(),A)
                   V=np.dot(A.transpose(),V)
                   B=np.dot(np.asmatrix(N)**-1,V)
                   exx=B[0]
                   eyy=B[4]
                   exy=(B[1]+B[3])/2
                   rotation=(B[1]-B[3])/2
                   dilatacion=exx+eyy
                   cizalla1=exx-eyy
                   cizalla2=2*exy
                   maxcizalla=np.sqrt(cizalla1**2+cizalla2**2)
                   thetastrain=.5*np.arctan(-cizalla2/cizalla1) + np.pi/2
                   emax=(dilatacion+maxcizalla)/2
                   emin=(dilatacion-maxcizalla)/2
                   maxgeo=max(np.abs(emax),np.abs(emin))
                   #este, norte, elevacion, delta gamma emax emin fi-strain max-defor-geo inclinacion fi-inclinacion
                   D=np.append(D,[[x0, y0, B[2]*1000, B[5]*1000, 0, dilatacion, maxcizalla, emax, emin, thetastrain,  maxgeo, 0, 0, rotation]],axis=0)
                   fidout.write('%-8.2f,%-8.2f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f,%-8.5f\n'%\
    				(x0,y0,B[2]*1000, B[5]*1000,emax*10**9,emin*10**9,thetastrain*180.0/np.pi,rotation*10**9,dilatacion*10**9,maxcizalla*10**9,maxgeo*10**9,np.abs(emax*10**9),np.abs(emin*10**9),90+thetastrain*180.0/np.pi,180+thetastrain*180.0/np.pi,270+thetastrain*180.0/np.pi))
        fidout.close()
        self.carga_fidout()
                       
    #...Creation of stress-strain tensors and vectors displacement sections...#

    #Displacement Vectors Layer
    def select_vector(self):
        self.iface.messageBar().pushMessage(
            "Drawing Arrows", "Successfully Done",
            level=Qgis.Success, duration=2)
      
        capa_activa = self.iface.activeLayer() 
        capa_activa_CRS=self.iface.activeLayer().crs().authid()
        
        params =  {'INPUT': capa_activa, 'STYLE' : self.plugin_dir + '/qml/vector_elip.qml'} #Arrows Style with ellipse error
        results = processing.run("qgis:setstyleforvectorlayer", params) 

    #Stress-Strain Tensors Layer
    def select_tensor(self):
        self.iface.messageBar().pushMessage(
            "Drawing tensors", "Successfully Done",
            level=Qgis.Success, duration=2)
      
        capa_activa = self.iface.activeLayer() 
        capa_activa_CRS=self.iface.activeLayer().crs().authid()
        
        params =  {'INPUT': capa_activa, 'STYLE' : self.plugin_dir + '/qml/tensor.qml'} # Strain Tensors Styles with extension & compression condition
        results = processing.run("qgis:setstyleforvectorlayer", params) 

    #Rasters layers creation (Maximum geodetic deformation, shear deformation, dilatation & rotation)
    def select_interpola(self):
        self.iface.messageBar().pushMessage(
            "Drawing Graph", "Successfully done",
            level=Qgis.Success, duration=5)

        Zfield=self.dlg.column_interpola.currentText()
        
        if (Zfield == 'ROTATION'):
                Zfield_value = 7
        elif (Zfield == 'DILATATION'):
                Zfield_value = 8
        elif (Zfield == 'SHEAR'):
                Zfield_value = 9
        elif (Zfield == 'MAXGEO'):
                Zfield_value = 10

        #Active layer (Qgis box)
        capa_activa = self.iface.activeLayer() 
        extension = capa_activa.extent()
        
        ly_xmin = extension.xMinimum()
        ly_xmax = extension.xMaximum()
        ly_ymin = extension.yMinimum()
        ly_ymax = extension.yMaximum()

        #String that stores the coordinates
        ly_coords = "%f,%f,%f,%f" %(ly_xmin, ly_xmax, ly_ymin, ly_ymax)
        
        #Active layer (second instance)
        capa_activa_CRS=self.iface.activeLayer().crs().authid()
        directorio_salida, fich_salida = os.path.split(self.fidout_name)

        #Selection of basic parameters, active layer, interpolation metod & column to draw the raster layer
        params =  { 'DISTANCE_COEFFICIENT' : 1, 'EXTENT' : ly_coords + ' ['+capa_activa_CRS+']', 'INTERPOLATION_DATA' : 'file:///'+self.fidout_name+'?delimiter=,&xField=x&yField=y&crs='+capa_activa_CRS+'::~::0::~::'+str(Zfield_value)+'::~::0', 'OUTPUT' : self.dlg.lineEdit_out_raster.text(), 'PIXEL_SIZE' : 200 }
        
        #Function to draw the raster layer
        result = processing.run("qgis:tininterpolation", params)
        result_layer = result['OUTPUT']
        uri = result_layer
        capa_interpolada = self.iface.addRasterLayer(uri,Zfield,"gdal")

    #Function to save the raster layer
    def select_output_raster(self):
      filename, _filter = QFileDialog.getSaveFileName(
        self.dlg, "Save image as","", '*.tiff') 
      self.dlg.lineEdit_out_raster.setText(filename)
    
    #Connection to panel buttons (Qt-Designer) and final functions
    def run(self):
        if self.first_start == True:
            self.first_start = False
            self.dlg = StrainStressAppDialog()
            self.dlg.setWindowFlags(Qt.WindowCloseButtonHint | Qt.WindowMinMaxButtonsHint)
            self.toolRectangleMap = RectangleMapTool(self.canvas, self.dlg)
            self.dlg.pushButton_2.clicked.connect(self.select_input_file)
            self.dlg.pushButton.clicked.connect(self.select_output_file)
            self.dlg.pushButton_3.clicked.connect(self.select_MREF)
            self.dlg.ejecutar.clicked.connect(self.select_ejecutar)
            self.dlg.push_btn_interpola.clicked.connect(self.select_interpola)
            self.dlg.pushButton_file_raster.clicked.connect(self.select_output_raster)
            self.dlg.start_tensor.clicked.connect(self.select_tensor)
            self.dlg.start_vector.clicked.connect(self.select_vector)
        
        layers = QgsProject.instance().layerTreeRoot().children()
        self.dlg.comboBox.clear()
        self.dlg.fich_datos.clear()
        self.dlg.fich_out.clear()
        self.dlg.comboBox.addItems([layer.name() for layer in layers])
        self.dlg.show()
        result = self.dlg.exec_()

        if result:
          filename = self.dlg.fich_out.text()
          with open(filename, 'w') as output_file:
            selectedLayerIndex = self.dlg.comboBox.currentIndex()
            selectedLayer = layers[selectedLayerIndex].layer()
            fieldnames = [field.name() for field in selectedLayer.fields()]
            line = ','.join(name for name in fieldnames) + '\n'
            output_file.write(line)

            for f in selectedLayer.getFeatures():
              line = ','.join(str(f[name]) for name in fieldnames) + '\n'
              output_file.write(line)
          self.iface.messageBar().pushMessage(
            "Successfully done", "files written in " + filename,
            level=Qgis.Success, duration=3)
    pass
